//
//  ConfigurarLlistaTableViewController.swift
//  LloguersTuristics
//
//  Created by Manel Juarez on 2/5/15.
//  Copyright (c) 2015 Manel Juarez. All rights reserved.
//

import UIKit
import CoreData

class ConfigurarLlistaTableViewController: UITableViewController, NSFetchedResultsControllerDelegate {

    // Obtenir el Context per treballar amb el Core Data
    let moContext = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
   
    // Array d'objectes NSManagedObject de la entity Configuracio (Classe Configuracio)
    var moConfiguracio = [Configuracio]()

    // El controlador que obté la informació (NSFetchedResultsController)
    // S'inicialitza de forma passiva "lazy", és dir es carregarà tot el primer cop que es faci referencia a la variable. Carregarà tots els registres de configuració de la taxa turística de la base de dades
    // L'ordenació és obligatoria i ho fem per Data.
    var fetchedResultsController: NSFetchedResultsController {
        
        // Retornem ell mateix si ja està creat (Patró Singleton)
        if self._fetchedResultsController != nil {
            return self._fetchedResultsController!
        }
        
        // Carregarem les dades de configuració de taxes, ordenades per data descendent
        let entity = NSEntityDescription.entityForName("Configuracio", inManagedObjectContext: moContext)
        let sort1 = NSSortDescriptor(key: "datavalidesa", ascending: false)
        let sort2 = NSSortDescriptor(key: "zona", ascending: true)
        let request = NSFetchRequest()
        request.entity = entity
        request.sortDescriptors = [sort1,sort2]
        
        // Aquesta llista no s'agrupa per seccions, per tant el sectionNameKeyPath es posa a nil
        let aFetchedResultsController = NSFetchedResultsController(fetchRequest: request, managedObjectContext: moContext, sectionNameKeyPath: nil, cacheName: nil)
        
        // El delegate per gestionar les actualitzacions serà aquest mateix controller
        aFetchedResultsController.delegate = self
        
        // Guardem els resultats
        self._fetchedResultsController = aFetchedResultsController
        
        // Llancem la primera petició de dades, controlant els errors
        var e: NSError?
        if !self._fetchedResultsController!.performFetch(&e) {
            println(NSLocalizedString("Error loading data:", comment: "Errors control") + " \(e!.localizedDescription)")
            abort();
        }
        
        // Retornem els resultats
        return self._fetchedResultsController!
    }
    // Variable per gestionar el patró singleton
    var _fetchedResultsController: NSFetchedResultsController?

    
    override func viewDidLoad() {
        super.viewDidLoad()
    }

    // Fem servir el control refresh "pull down" i que recarregui la informació de la llista
    @IBAction func refrescarLlista(sender: UIRefreshControl) {
        // Provocar que el tableView torni a demanar les dades
        tableView.reloadData()
        // Amagar la imatge "spinning-carregant"
        sender.endRefreshing()
    }

    // MARK: - Table view data source

    // Retornar el total de seccions
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        // Demanem al controller que gestiona les dades quantes seccions hi ha
        if let sections = self.fetchedResultsController.sections {
            return sections.count
        }
        return 0
    }
    
    // Retornar el total de registres en una secció
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // Demanem al controller que gestiona les dades quants registres hi ha
        if let sections = self.fetchedResultsController.sections {
            let currentSection = sections[section] as NSFetchedResultsSectionInfo
            return currentSection.numberOfObjects
        }
        return 0
    }

    // Petició de dades per una cel·la concreta
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        // Demanar una cel·la que estigui lliure o sinó en crearà una de nova del tipus "TaxaCell"
        let cell = tableView.dequeueReusableCellWithIdentifier("TaxaCell", forIndexPath: indexPath) as UITableViewCell
        
        // Configurar la cel·la
        self.configurarCell(cell, atIndexPath: indexPath)
        return cell
    }
    
    // Configurar (carregar dades) a una cel·la a partir d'un registre determinat (NSIndexPath) del NSFetchedResultsController
    func configurarCell(cell: UITableViewCell,
        atIndexPath indexPath: NSIndexPath) {
            
            // Carregar i fer el cast del la configuració de taxa turística seleccionada
            let item = self.fetchedResultsController.objectAtIndexPath(indexPath) as Configuracio
            
            // Com que són cel·les tipus "custom" amb 5 labels, cadascun té un número "tag" diferent.
            var lblDataValidesa : UILabel? = self.view.viewWithTag(10) as? UILabel;
            var lblZona : UILabel? = self.view.viewWithTag(40) as? UILabel;
            var lblTaxaPersonaNit : UILabel? = self.view.viewWithTag(50) as? UILabel;
            
            // Mostrar data validesa a la cel·la
            // La data de la factura s'ha de formatar per poder-la mostrar
            var dateFormatter: NSDateFormatter = NSDateFormatter()
            dateFormatter.locale = NSLocale.currentLocale()
            dateFormatter.dateStyle = NSDateFormatterStyle.MediumStyle
            lblDataValidesa?.text = dateFormatter.stringFromDate(item.datavalidesa) as String?
            
            // Cal tenir en compte la localització (per exemple en català/español el decimal és una coma, no un punt. Afagem el currentLocale de l'aparell de l'usuari
            var formatter = NSNumberFormatter()
            formatter.locale = NSLocale.currentLocale()
            formatter.numberStyle = NSNumberFormatterStyle.CurrencyStyle
            
            // Mostrar zona de la taxa
            lblZona?.text = item.zona.stringValue
            // Mostrar taxa per persona/nit
            lblTaxaPersonaNit?.text = formatter.stringFromNumber(item.taxapersonanit)
    }

    // Assignar títol i estil (color) al fons de la única secció
    override func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        var header :UITableViewHeaderFooterView = UITableViewHeaderFooterView()
        header.textLabel.text = NSLocalizedString("Tourism Tax Settings", comment: "Títol llista taxes configuració")
        header.contentView.backgroundColor = UIColor(red: 108.0/255.0, green: 226.0/255.0, blue: 108.0/255.0, alpha: 0.5)
        return header
    }
    
    //
    // Gestió de les actualitzacions del "fetched results controller delegate"
    //
    
    // Es crida al principi
    // Serveix per assegurar que totes les actualitzacions apliquen l'animació a la vegada
    func controllerWillChangeContent(controller: NSFetchedResultsController) {
        self.tableView.beginUpdates()
    }
    
    // Es crida quan:
    // Es crea un nou model (nou habitatge)
    // S'actualitza un model ja existent (modificar/editar habitatge)
    // S'elimina un model (esborrar habitatge)
    func controller(controller: NSFetchedResultsController,
        didChangeObject object: AnyObject,
        atIndexPath indexPath: NSIndexPath,
        forChangeType type: NSFetchedResultsChangeType,
        newIndexPath: NSIndexPath) {
            switch type {
            case .Insert:
                self.tableView.insertRowsAtIndexPaths([newIndexPath], withRowAnimation: .Fade)
            case .Update:
                let cell = self.tableView.cellForRowAtIndexPath(indexPath)
                self.configurarCell(cell!, atIndexPath: indexPath)
                self.tableView.reloadRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
                // No permetem moure (reordenar habitatges) per tant aquesta part no cal
                //case .Move:
                //    self.tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
                //    self.tableView.insertRowsAtIndexPaths([newIndexPath], withRowAnimation: .Fade)
            case .Delete:
                self.tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
            default:
                return
            }
    }
    
    // Es crida al final
    // Indica al UITableView que s'han completat les actualitzacions
    func controllerDidChangeContent(controller: NSFetchedResultsController) {
        self.tableView.endUpdates()
    }
    
    
    // Controlar l'eliminació d'un registre fent tap+swipe esquerra (apareix botó "eliminar")
    override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        
        if editingStyle == .Delete {
            // Esborrar habitatge del core data
            if let configSelec = self.fetchedResultsController.objectAtIndexPath(indexPath) as? Configuracio {
                // Esborrar del context core data
                moContext.deleteObject(configSelec)
                // Guardar context
                var errorGuardar: NSError? = nil
                if !moContext.save(&errorGuardar) {
                    println(NSLocalizedString("Error deleting tax definition:", comment: "Errors control") + "  \(errorGuardar!.localizedDescription)")
                    abort();
                }
            }
        }
    }
    
    
    // MARK: - Navigation

    //
    // Atendre les peticions dels Segue que tenim definits a l'storyboard
    //
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        var destination = segue.destinationViewController as? UIViewController
        
        // La vista per editar un ingrés està embebida dins un UINavigationController
        // Per tant cal allargar el segue i saltar el UINav
        if let navcon = destination as? UINavigationController {
            destination = navcon.visibleViewController
        }
        
        // Si el segue apunta a la fitxa per editar ingrés
        if let vc = destination as? ConfigurarTaxaEditarTableViewController {
            if let identifier = segue.identifier {
                switch identifier {
                case "NovaTaxa":
                    vc.moTaxaActual = nil
                    break
                case "EditarTaxa":
                    // Obtenir la fila del tableview que està seleccionada
                    if let indexPath = self.tableView.indexPathForSelectedRow() {
                        // Obtenir l'objecte ingrés que correspon a la fila seleccionada
                        let taxaSelec = self.fetchedResultsController.objectAtIndexPath(indexPath) as? Configuracio
                        // Assignar valors al MVC destí
                        vc.moTaxaActual = taxaSelec
                    }
                    break
                default:
                    break
                }
            }
        }
    }
    
    
    // Funció per atendre els unwind segue (els segue de retorn)
    @IBAction func unwindEditarTaxa(segue: UIStoryboardSegue) {
        var vcTaxaEditar = segue.sourceViewController as ConfigurarTaxaEditarTableViewController
        var moTaxaEditat: Configuracio? = vcTaxaEditar.moTaxaActual
        
        if let identifier = segue.identifier {
            switch identifier {
            case "TaxaCancelar":
                // Si l'usuari ha cancelat l'edició de la taxa no cal fer res més
                break
            case "TaxaGuardar":
                // Provocar que el tableView torni a demanar les dades
                tableView.reloadData()
                break
            default:
                break
            }
        }
    }
    
}
