//
//  DespesesLlistaTableViewController.swift
//  LloguersTuristics
//
//  Created by Manel Juarez on 6/4/15.
//  Copyright (c) 2015 Manel Juarez. All rights reserved.
//

import UIKit
import CoreData

class DespesesLlistaTableViewController: UITableViewController {

    // Obtenir el Context per treballar amb el Core Data
    let moContext = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
    
    // Objecte que conté les dades del habitatge actual que estem mostrant.
    var moHabitatgeActual: Habitatges?
    
    // Array de objectes NSManagedObject de la entity Depesa (Classe Despeses)
    // És el Model d'aquest MVC
    var moDespeses = [Despeses]()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Afegir botons de navegació per poder tornar enrera
        navigationItem.leftBarButtonItem = splitViewController?.displayModeButtonItem()
        navigationItem.leftItemsSupplementBackButton = true
        
        // Mostrar dades habitatge
        carregarDadesDespesesHabitatge()
    }
    
    //
    // Fem servir el control refresh per simular un "pull down" i que recarregui la informació de la llista
    //
    func carregarDadesDespesesHabitatge() {
        // mostrar imatge "carregant"
        refreshControl?.beginRefreshing()
        refrescarLlista(refreshControl!)
    }
    
    @IBAction func refrescarLlista(sender: UIRefreshControl) {
        // Carregar les despeses que tenim a l'objecte de l'habitatge actual
        // Cal convertir el tipus NSSet de les despeses (relació un-a-molts del core data)
        // a un tipus Array. "allOjbects" ho fa directament.
        moDespeses = moHabitatgeActual?.despeses.allObjects as [Despeses]
        // Ordenar les despeses per data de factura descendent
        moDespeses.sort({ $0.datafactura.compare($1.datafactura) == NSComparisonResult.OrderedDescending})
        
        // Provocar que el tableView torni a demanar les dades
        tableView.reloadData()
        
        // Amagar la imatge "spinning-carregant"
        sender.endRefreshing()
    }
    
    // MARK: - Table view data source
    
    // Retornar el total de seccions
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    // Retornar el total de files per la secció indicada
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return moDespeses.count
    }
    
    // Configurar la informació que es mostra en una cel·la (estàtica - custom)
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("DespesaCell", forIndexPath: indexPath) as UITableViewCell
        
        // Configurar la cel·la per mostrar informació de la despesa
        // Com que són cel·les tipus "custom" amb 3 labels, cadascun té un número "tag" diferent.
        var lblDataFactura : UILabel? = self.view.viewWithTag(10) as? UILabel;
        var lblConcepte : UILabel? = self.view.viewWithTag(20) as? UILabel;
        var lblImportPagat : UILabel? = self.view.viewWithTag(30) as? UILabel;
        
        // La data de la factura s'ha de formatar per poder-la mostrar
        var dateFormatter: NSDateFormatter = NSDateFormatter()
        dateFormatter.locale = NSLocale.currentLocale()
        dateFormatter.dateStyle = NSDateFormatterStyle.MediumStyle
        lblDataFactura?.text = dateFormatter.stringFromDate(moDespeses[indexPath.row].datafactura) as String?
        
        lblConcepte?.text = moDespeses[indexPath.row].concepte
        
        // Cal tenir en compte la localització (per exemple en català/español el decimal és una coma, no un punt. Afagem el currentLocale de l'aparell de l'usuari
        var formatter = NSNumberFormatter()
        formatter.locale = NSLocale.currentLocale()
        formatter.numberStyle = NSNumberFormatterStyle.CurrencyStyle
        lblImportPagat?.text = formatter.stringFromNumber(moDespeses[indexPath.row].importpagat)
        
        return cell
    }
    
    // Assignar títol i estil (color) al fons de la única secció
    override func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        var header :UITableViewHeaderFooterView = UITableViewHeaderFooterView()
        header.textLabel.text = moHabitatgeActual?.nom
        header.contentView.backgroundColor = UIColor(red: 108.0/255.0, green: 226.0/255.0, blue: 108.0/255.0, alpha: 0.5)
        return header
    }
    
    // Controlar la funció d'esborrar un registre
    override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        if editingStyle == .Delete {
            // Esborrar ingrés del core data
            if let despesa = moDespeses[indexPath.row] as Despeses? {
                // Esborrar del context core data
                moContext.deleteObject(despesa)
                
                // Guardar context
                var errorGuardar: NSError? = nil
                if !moContext.save(&errorGuardar) {
                    println(NSLocalizedString("Error deleting expense:", comment: "Errors control") + " \(errorGuardar!.localizedDescription)")
                    abort();
                } else {
                    // Esborrar del array de despeses del controller (Model)
                    moDespeses.removeAtIndex(indexPath.row)
                    // Delete the row from the data source
                    tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
                }
            }
        }
    }
    
    
    // MARK: - Navigation
    
    //
    // Atendre les peticions dels Segue que tenim definits a l'storyboard
    //
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        var destination = segue.destinationViewController as? UIViewController
        
        // La vista per editar una despesa està embebida dins un UINavigationController
        // Per tant cal allargar el segue i saltar el UINav
        if let navcon = destination as? UINavigationController {
            destination = navcon.visibleViewController
        }
        
        // Si el segue apunta a la fitxa per editar despesa
        if let vc = destination as? DespesaEditarTableViewController {
            if let identifier = segue.identifier {
                switch identifier {
                case "NovaDespesa":
                    vc.moHabitatgeActual = self.moHabitatgeActual
                    vc.moDespesaActual = nil
                    break
                case "EditarDespesa":
                    // Obtenir la fila del tableview que està seleccionada
                    if let indexPath = self.tableView.indexPathForSelectedRow() {
                        // Obtenir l'objecte despesa que correspon a la fila seleccionada
                        let despesaSelec = self.moDespeses[indexPath.row]
                        // Assignar valors al MVC destí
                        vc.moHabitatgeActual = self.moHabitatgeActual
                        vc.moDespesaActual = despesaSelec
                    }
                    break
                default:
                    break
                }
            }
        }
    }
    
    
    // Funció per atendre els unwind segue (els segue de retorn)
    @IBAction func unwindEditarDespesa(segue: UIStoryboardSegue) {
        var vcDespesaEditar = segue.sourceViewController as DespesaEditarTableViewController
        var moDespesaEditada: Despeses? = vcDespesaEditar.moDespesaActual
        
        if let identifier = segue.identifier {
            switch identifier {
            case "DespesaCancelar":
                // Si l'usuari ha cancelat l'edició de la despesa no cal fer res més
                break
            case "DespesaGuardar":
                // Refrescar llista de despeses
                carregarDadesDespesesHabitatge()
                break
            default:
                break
            }
        }
    }

}
