//
//  IngressosLlistaTableViewController.swift
//  LloguersTuristics
//
//  Created by Manel Juarez on 3/4/15.
//  Copyright (c) 2015 Manel Juarez. All rights reserved.
//

import UIKit
import CoreData

class IngressosLlistaTableViewController: UITableViewController {

    // Obtenir el Context per treballar amb el Core Data
    let moContext = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!

    // Objecte que conté les dades del habitatge actual que estem mostrant.
    var moHabitatgeActual: Habitatges?
    
    // Array de objectes NSManagedObject de la entity Ingres (Classe Ingressos)
    // És el Model d'aquest MVC
    var moIngressos = [Ingressos]()
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Afegir botons de navegació per poder tornar enrera
        navigationItem.leftBarButtonItem = splitViewController?.displayModeButtonItem()
        navigationItem.leftItemsSupplementBackButton = true
        
        // Mostrar dades habitatge
        carregarDadesIngressosHabitatge()        
    }

    //
    // Fem servir el control refresh per simular un "pull down" i que recarregui la informació de la llista
    //
    func carregarDadesIngressosHabitatge() {
        // mostrar imatge "carregant"
        refreshControl?.beginRefreshing()
        refrescarLlista(refreshControl!)
    }

    @IBAction func refrescarLlista(sender: UIRefreshControl) {
        // Carregar els ingressos que tenim a l'objecte de l'habitatge actual
        // Cal convertir el tipus NSSet dels ingressos (relació un-a-molts del core data)
        // a un tipus Array. "allOjbects" ho fa directament.
        moIngressos = moHabitatgeActual?.ingressos.allObjects as [Ingressos]
        // Ordenar els ingressos per data d'entrada descendent
        moIngressos.sort({ $0.dataentrada.compare($1.dataentrada) == NSComparisonResult.OrderedDescending})
        
        // Provocar que el tableView torni a demanar les dades
        tableView.reloadData()
        
        // Amagar la imatge "spinning-carregant"
        sender.endRefreshing()
    }
    
    // MARK: - Table view data source

    // Retornar el total de seccions
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    // Retornar el total de files per la secció indicada
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return moIngressos.count
    }
    
    // Configurar la informació que es mostra en una cel·la (estàtica - custom)
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("IngresCell", forIndexPath: indexPath) as UITableViewCell
        
        // Configurar la cel·la per mostrar informació de l'ingrés
        // Com que són cel·les tipus "custom" amb 3 labels, cadascun té un número "tag" diferent.
        var lblData : UILabel? = self.view.viewWithTag(10) as? UILabel;
        var lblNomLlogater : UILabel? = self.view.viewWithTag(20) as? UILabel;
        var lblImportCobrat : UILabel? = self.view.viewWithTag(30) as? UILabel;

        // La data de l'ingrés s'ha de formatar per poder-la mostrar
        var dateFormatter: NSDateFormatter = NSDateFormatter()
        dateFormatter.locale = NSLocale.currentLocale()
        dateFormatter.dateStyle = NSDateFormatterStyle.MediumStyle
        lblData?.text = dateFormatter.stringFromDate(moIngressos[indexPath.row].dataentrada) as String?
        
        lblNomLlogater?.text = moIngressos[indexPath.row].nomllogater
        
        // Cal tenir en compte la localització (per exemple en català/español el decimal és una coma, no un punt. Afagem el currentLocale de l'aparell de l'usuari
        var formatter = NSNumberFormatter()
        formatter.locale = NSLocale.currentLocale()
        formatter.numberStyle = NSNumberFormatterStyle.CurrencyStyle
        lblImportCobrat?.text = formatter.stringFromNumber(moIngressos[indexPath.row].importcobrat)
        
        return cell
    }

    // Assignar títol i estil (color) al fons de la única secció
    override func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        var header :UITableViewHeaderFooterView = UITableViewHeaderFooterView()
        header.textLabel.text = moHabitatgeActual?.nom
        header.contentView.backgroundColor = UIColor(red: 108.0/255.0, green: 226.0/255.0, blue: 108.0/255.0, alpha: 0.5)
        return header
    }

    // Controlar la funció d'esborrar un registre
    override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        if editingStyle == .Delete {
            // Esborrar ingrés del core data
            if let ingres = moIngressos[indexPath.row] as Ingressos? {
                // Esborrar del context core data
                moContext.deleteObject(ingres)
                
                // Guardar context
                var errorGuardar: NSError? = nil
                if !moContext.save(&errorGuardar) {
                    println(NSLocalizedString("Error deleting income:", comment: "Errors control") + " \(errorGuardar!.localizedDescription)")
                    abort();
                } else {
                    // Esborrar del array d'ingressos del controller (Model)
                    moIngressos.removeAtIndex(indexPath.row)
                    // Delete the row from the data source
                    tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
                }
            }
        }
    }
    

    // MARK: - Navigation
    
    //
    // Atendre les peticions dels Segue que tenim definits a l'storyboard
    //
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        var destination = segue.destinationViewController as? UIViewController
        
        // La vista per editar un ingrés està embebida dins un UINavigationController
        // Per tant cal allargar el segue i saltar el UINav
        if let navcon = destination as? UINavigationController {
            destination = navcon.visibleViewController
        }
        
        // Si el segue apunta a la fitxa per editar ingrés
        if let vc = destination as? IngresEditarTableViewController {
            if let identifier = segue.identifier {
                switch identifier {
                case "NouIngres":
                    vc.moHabitatgeActual = self.moHabitatgeActual
                    vc.moIngresActual = nil
                    break
                case "EditarIngres":
                    // Obtenir la fila del tableview que està seleccionada
                    if let indexPath = self.tableView.indexPathForSelectedRow() {
                        // Obtenir l'objecte ingrés que correspon a la fila seleccionada
                        let ingresSelec = self.moIngressos[indexPath.row]
                        // Assignar valors al MVC destí
                        vc.moHabitatgeActual = self.moHabitatgeActual
                        vc.moIngresActual = ingresSelec
                    }
                    break
                default:
                    break
                }
            }
        }
    }
   
    
    // Funció per atendre els unwind segue (els segue de retorn)
    @IBAction func unwindEditarIngres(segue: UIStoryboardSegue) {
        var vcIngresEditar = segue.sourceViewController as IngresEditarTableViewController
        var moIngresEditat: Ingressos? = vcIngresEditar.moIngresActual
    
        if let identifier = segue.identifier {
            switch identifier {
            case "IngresCancelar":
                // Si l'usuari ha cancelat l'edició de l'ingrés no cal fer res més
                break
            case "IngresGuardar":
                // Refrescar llista ingressos
                carregarDadesIngressosHabitatge()
                break
            default:
                break
            }
        }
    }

}
